# This Build.PL for JSON-Schema-Tiny was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.020.
use strict;
use warnings;

if ("$]" < 5.038) {
  die "This distribution will not install where builtin::Backport exists.\n"
    if eval { +require builtin::Backport; 1 };
}

use 5.020;
use Module::Build::Tiny 0.034;
Build_PL();

